/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.web.help;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import dk.vajhoej.vms.web.FunctionModule;
import dk.vajhoej.vms.web.Result;
import dk.vajhoej.vms.web.Util;

/**
 * Class HelpModule exposes VMS HELP via LBR$ routines.
 */
public class HelpModule implements FunctionModule {
	private static final String DELIM = "Additional information available:";
	private static String getText(String helpoutput) {
		String[] areas = helpoutput.split(DELIM);
		return areas[0];
	}
	private static String[] getSubTopics(String helpoutput) {
		String[] areas = helpoutput.split(DELIM);
		if(areas.length > 1) {
			return areas[1].split("\\s+");
		} else {
			return new String[0];
		}
	}
	private static class HelpOutput {
		private String text;
		private String[] subTopics;
		public HelpOutput(String text, String[] subTopics) {
			this.text = text;
			this.subTopics = subTopics;
		}
		public String getText() {
			return text;
		}
		public String[] getSubTopics() {
			return subTopics;
		}
	}
	private static HelpOutput getHelp(String[] topics) throws UnsupportedEncodingException {
		LBRWrapper lbr = new LBRWrapper();
		lbr.init();
		String helpoutput = lbr.get(topics);
		lbr.close();
		return new HelpOutput(getText(helpoutput), getSubTopics(helpoutput));
	}
	private static String toHTML(String txt) throws IOException {
		StringBuilder sb = new StringBuilder();
		BufferedReader br = new BufferedReader(new StringReader(txt));
		sb.append("<p>\r\n");
		String line;
		while((line = br.readLine()) != null) {
			if(line.trim().equals("")) {
				sb.append("</p>\r\n<p>\r\n");
			} else {
				sb.append(line);
				sb.append("\r\n");
			}
		}
		br.close();
		sb.append("</p>\r\n");
		return sb.toString();
	}
	private static final String DEFAULT_TOPIC = "HELP";
	private static String format(String query, HelpOutput data) throws IOException {
		String maintmpl = Util.loadText("main.tmpl");
		String linktmpl = Util.loadText("link.tmpl");
		String res = maintmpl;
		// main text
		res = res.replace("%TEXT%", data.getText()).replace("%HTML%", toHTML(data.getText())).replace("%TOPIC%", query.replace("+", " "));
		// sub topics
		StringBuilder sb = new StringBuilder();
		for(String subtopic : data.getSubTopics()) {
			if(subtopic.length() > 0) {
				String link;
				if(query.equals(DEFAULT_TOPIC)) {
					link = subtopic;
				} else {
					link = query + "+" + subtopic;
				}
				sb.append(linktmpl.replace("%SUBTOPIC%", subtopic).replace("%LINK%", link));
			}
		}
		res = res.replace("%SUBTOPICS%", sb.toString());
		// back link
		String backlink;
		int lastsep = query.lastIndexOf("+");
		if(lastsep > 0) {
			backlink = query.substring(0, lastsep);
		} else {
			backlink = DEFAULT_TOPIC;
		}
		res = res.replace("%BACKLINK%", linktmpl.replace("%SUBTOPIC%", "Back").replace("%LINK%", backlink));
		// top link
		res = res.replace("%TOPLINK%", linktmpl.replace("%SUBTOPIC%", "Top").replace("%LINK%", DEFAULT_TOPIC));
		//
		return  res;
	}
	public String getContext() {
		return "help";
	}
	private Map<String,HelpOutput> cache = new HashMap<String,HelpOutput>();
	public Result getResponse(String remurl, String query) {
		try {
			if(query == null || query.equals("")) query = DEFAULT_TOPIC;
			HelpOutput data;
			if(cache.containsKey(query)) {
				data = cache.get(query);
			} else {
				String[] topics = query.split("\\+");
				if(topics.length> 5) {
					return new Result(false, "Bad topics specified", null, null);	
				}
				data = getHelp(topics);
				cache.put(query, data);
			}
			String res = format(query, data);
			return new Result(true, null, "text/html", res.getBytes("UTF-8"));
		} catch (Exception e) {
			return new Result(false, "Error getting help information: " + e.getMessage(), null, null);
		}
	}
}
